@@ -23,7 +23,7 @@ ctStanContinuousPars <- function(fit,
 
   if(!'ctStanFit' %in% class(fit)) stop('Not an object of class ctStanFit')
   
-  e<-fit$stanfit$transformedpars #first dim of subobjects is iter, 2nd subjects
+  e<-ctExtract(fit,cores=1) #Qfit$stanfit$transformedpars #first dim of subobjects is iter, 2nd subjects
   niter=dim(e$pop_DRIFT)[1]
 
   
@@ -40,7 +40,6 @@ ctStanContinuousPars <- function(fit,
 
   out <- list()
   for(matname in (mats)){
-
     try({
       calcfuncargs$collapsemargin = 1
     calcfuncargs$collapsefunc=calcfunc
@@ -56,7 +55,7 @@ ctStanContinuousPars <- function(fit,
     nlatent <- nrow(out$CINT)
     out$T0MEANS <- out$T0MEANS[1:nlatent,1,drop=FALSE]
     out$DRIFT <- out$DRIFT[1:nlatent,1:nlatent,drop=FALSE]
-    # out$T0VAR <- out$T0VAR[1:nlatent,1:nlatent,drop=FALSE]
+    out$T0VAR <- out$T0VAR[1:nlatent,1:nlatent,drop=FALSE]
     out$T0cov <- out$T0cov[1:nlatent,1:nlatent,drop=FALSE]
   }
   
@@ -65,13 +64,14 @@ ctStanContinuousPars <- function(fit,
   tdn=fit$ctstanmodel$TDpredNamesQ
   dimnames(out$DRIFT)=list(ln,ln)
   dimnames(out$DIFFUSIONcov)=list(ln,ln)
+  dimnames(out$DIFFUSION)=list(ln,ln)
   dimnames(out$T0cov)=list(ln,ln)
   dimnames(out$asymDIFFUSION)=list(ln,ln)
   rownames(out$CINT)=ln
   rownames(out$MANIFESTMEANS)=mn
   rownames(out$T0MEANS)=ln
   
-  # dimnames(out$T0VAR)=list(ln,ln)
+  dimnames(out$T0VAR)=list(ln,ln)
   dimnames(out$asymDIFFUSION)=list(ln,ln)
   dimnames(out$LAMBDA)=list(mn,ln)
 
@@ -88,7 +88,7 @@ ctStanContinuousPars <- function(fit,
     out$TDPREDEFFECT<-out$TDPREDEFFECT
   }
 
-  out$MANIFESTVAR <- NULL ; out$DIFFUSION <- NULL; out$T0VAR <- NULL
+  out$MANIFESTVAR <- NULL ; 
   names(out)[names(out) %in% 'asymDIFFUSION'] <- 'asymDIFFUSIONcov'
   
   
