@@ -6,14 +6,10 @@ ctStanData <- function(ctm, datalong,optimize,derrind='all'){
   }
   datalong <- datalong[,c(ctm$timeName,ctm$subjectIDname,
     ctm$manifestNames,ctm$TDpredNames,ctm$TIpredNames)]
-  #start data section
-  # if('data.table' %in% class(datalong) || 'tbl' %in% class(datalong)) 
-  
+
   
   nsubjects <- length(unique(datalong[, ctm$subjectIDname])) 
-  
-  #create random effects indices for each matrix
-  
+
   mats <- ctStanMatricesList()
   
   #simply exponential?
@@ -27,7 +23,7 @@ ctStanData <- function(ctm, datalong,optimize,derrind='all'){
       variables: ', paste0(c(ctm$manifestNames,ctm$TDpredNames,ctm$TIpredNames)[
         which(!c(ctm$manifestNames,ctm$TDpredNames,ctm$TIpredNames) %in% colnames(datalong))], ', '),' not in data'))
   
-  if (!(ctm$subjectIDname %in% colnames(datalong))) stop(paste('id column', (ctm$subjectIDname), "not found in data"))
+  if(!(ctm$subjectIDname %in% colnames(datalong))) stop(paste('id column', (ctm$subjectIDname), "not found in data"))
   
   
   if(ctm$nopriors==FALSE){
@@ -55,7 +51,7 @@ ctStanData <- function(ctm, datalong,optimize,derrind='all'){
   
   
   
-
+  
   if( (nrow(ctm$t0varstationary) + nrow(ctm$t0meansstationary)) >0 && 
       length(c(ctm$modelmats$calcs$driftcint, ctm$modelmats$calcs$diffusion)) > 0) message('Stationarity assumptions based on initial states when using non-linear dynamics')
   
@@ -70,7 +66,7 @@ ctStanData <- function(ctm, datalong,optimize,derrind='all'){
   ndiffusion=length(derrind)
   
   
-  
+
   nindvarying <- max(ctm$modelmats$matsetup$indvarying)
   nparams <- max(ctm$modelmats$matsetup$param[ctm$modelmats$matsetup$when %in% c(0,-1)])
   nmatrices <- length(mats$base)
@@ -216,7 +212,7 @@ ctStanData <- function(ctm, datalong,optimize,derrind='all'){
   if(ctm$n.TIpred == 0) tipreds <- array(0,c(0,0))
   standata$tipredsdata <- as.matrix(tipreds)
   standata$nmissingtipreds <- as.integer(length(tipreds[tipreds== 99999]))
-
+  
   standata$ntipredeffects <- as.integer(ifelse(ctm$n.TIpred > 0, as.integer(max(ctm$modelmats$TIPREDEFFECTsetup)), 0))
   standata$TIPREDEFFECTsetup <- apply(ctm$modelmats$TIPREDEFFECTsetup,c(1,2),as.integer,.drop=FALSE)
   standata$tipredsimputedscale <- ctm$tipredsimputedscale
@@ -308,7 +304,7 @@ ctStanData <- function(ctm, datalong,optimize,derrind='all'){
   
   standata$subindices <- as.integer(unlist(subindices))[order(mats$base)]
   
-
+  
   #state dependence
   statedep <- rep(0L,as.integer(max(mats$all)))
   lhscalcs <- sapply(unique(unlist(ctm$modelmats$calcs)),function(x) gsub('=.*','',x))
@@ -411,7 +407,7 @@ ctStanData <- function(ctm, datalong,optimize,derrind='all'){
     standata$whenvecs[wheni,ms$param[ms$when %in% c(wheni,100) & ms$copyrow <=0 & ms$param > 0]] <- 
       as.integer(ms$param[ms$when  %in% c(wheni,100) & ms$copyrow <=0 & ms$param > 0]) #100 is for PARS - needed everywhere.
   }
-
+  
   # why was this in the code? when do we need the below line... ind varying based on states?
   # if(standata$intoverpop==1 && standata$nlatentpop > standata$nlatent){
   #   standata$whenvecs[5,ms$param[ms$when==0 & ms$param > 0 & ms$copyrow < 1 & (ms$indvarying > 0 | ms$tipred > 0)]] <- 
@@ -429,7 +425,48 @@ ctStanData <- function(ctm, datalong,optimize,derrind='all'){
   standata$whenmat[mc[names(mc) == 'MANIFESTcov'],] <- standata$whenmat[mc[names(mc) == 'MANIFESTVAR'],]
   
   standata$statedep[31:33] <- standata$statedep[c(4,5,8)]
-
+  
+  #laplace priors
+  standata$laplaceprior <- rep(0L,standata$nparams)
+  if(!is.null(ctm$laplaceprior)){
+    ms <- data.frame(standata$matsetup)
+    standata$laplaceprior[
+      ms$param[
+        ms$matrix %in% ctStanMatricesList()$all[names(ctStanMatricesList()$all) %in% ctm$laplaceprior] & 
+          ms$param > 0 & 
+          ms$row!=ms$col & 
+          ms$when==0 & 
+          ms$copyrow<1]
+    ] <- 1L
+  }
+  standata$laplaceprioronly <- ifelse(is.null(ctm$laplaceprioronly),0L,as.integer(ctm$laplaceprioronly))
+  
+  
+  #CINT non zero
+  ms <- data.frame(standata$matsetup)
+  CINTnonzero <- c()#1:standata$nlatent
+  for(i in 1:standata$nlatent){
+    ri=which(ms$matrix %in% ctStanMatricesList()$all[names(ctStanMatricesList()$all) %in% 'CINT'] & ms$row %in% i)
+    if(ms$param[ri]==0 && standata$matvalues[ri,'value']==0) next else CINTnonzero <- c(CINTnonzero,i)
+  }
+  standata$CINTnonzero <- array(as.integer(CINTnonzero))
+  standata$CINTnonzerosize <- length(CINTnonzero)
+  
+  #JAx different from DRIFT?
+  standata$JAxDRIFTequiv <- 1L
+  ml <- listOfMatrices(ctm$pars)
+  for(i in 1:nrow(ml$JAx)){
+    for(j in 1:ncol(ml$JAx)){
+      if(i <= nrow(ml$DRIFT) && j <= nrow(ml$DRIFT)){ #check drift equivalence
+        # if(i==j && !ctm$continuoustime && ml$JAx[i,j] %in% 1) ml$JAx[i,j] <- 
+        if(ml$JAx[i,j] != ml$DRIFT[i,j])   standata$JAxDRIFTequiv <- 0L
+      }
+      if(i > nrow(ml$DRIFT) || j > nrow(ml$DRIFT)){ #check drift equivalence
+        if(i != j && !ml$JAx[i,j] %in% 0)   standata$JAxDRIFTequiv <- 0L
+        if(i == j && !ml$JAx[i,j] %in% ifelse(ctm$continuoustime,0,1))   standata$JAxDRIFTequiv <- 0L
+      }
+    }
+  }
   
   return(standata)
 }
